
typedef struct
      {
  int   type;
  int   x, y;
  int   modifiers;
  int   button;
  int   key;
  char *string;
  void *usr;
  int wide;
  double flo0, flo1;
      }
    gfx_event_t;

enum
{
  GFXEV_BUTTON_PRESS=1, GFXEV_BUTTON_RELEASE,
  GFXEV_KEY_PRESS,      GFXEV_KEY_RELEASE,
  GFXEV_POINTER_MOVE,   GFXEV_QUIT, GFXEV_LAST,
};


/* The event routine returns an event from the queue. It waits a maximum
   of 'msecs' milliseconds for the event. If msecs is zero, it waits
   until an event is received.

   The return value is one if an event is returned. It's zero if the
   return was due to timeout.

   In case of an event, *msecs is updated to reflect the remaining
   wait time.  */

int gfx_event(unsigned int *msecs,gfx_event_t *Revent);

int gfx_init(char *name, int width, int height, char *title);

#define GFX_UPDATE (1<<0)
#define GFX_ALPHA  (1<<1)
#define GFX_ORDER  (1<<0)    // order returned coordinates
#define GFX_BACKGND (1<<2)   // use background color instead of pixel buffer
#define GFX_JUST_RIGHT (1<<3)
#define GFX_JUST_CENTER (1<<4)
#define GFX_CUT_LEFT (1<<5)
#define GFX_FOCUS (1<<6)
#define GFX_HORIZONTAL (1<<7)
#define GFX_VERTICAL   (1<<8)
#define GFX_NOICON     (1<<9)
#define GFX_MULTISELECT (1<<3)


void gfx_update(int x,int y,int width,int height);
void gfx_update_all();

enum
{ 
   GFX_K_PRT=1, 
   GFX_K_BACKSPACE,     GFX_K_TAB,      GFX_K_RETURN, GFX_K_ESCAPE,
   GFX_K_INSERT,        GFX_K_DELETE,   GFX_K_HOME,   GFX_K_END,
   GFX_K_PAGEUP,        GFX_K_PAGEDOWN,
   GFX_K_LEFT,          GFX_K_RIGHT,    GFX_K_DOWN,   GFX_K_UP,
   GFX_K_CONTROL=0x100, GFX_K_SHIFT= 0x200 
};


typedef struct { int iv1, iv2; char *sv; } TUPiis_t;


typedef struct
{
  char *str;
  unsigned int *icon_data, icon_width, icon_height;
  int iv;
  void *pv;
} TUPscip_t;



void gfx_geometry(unsigned int *W, unsigned int *H);
void gfx_get(unsigned int *buffer, int dst_w, int dst_h,
             int dst_x, int dst_y,
             int src_x, int src_y,
             int width, int height);
void gfx_fill_rect(int flags, int dst_x,int dst_y,
                   int w,int h, unsigned int color);
void gfx_draw_rect(int flags, int dst_x,int dst_y,
                   int w,int h, unsigned int color);
void gfx_put_graymap(int flags,
             unsigned char *buffer, int src_w, int src_h,
             int src_x, int src_y,
             int dst_x, int dst_y,
             int width, int height);
void gfx_put(int flags,
             unsigned int *buffer, int src_w, int src_h,
             int src_x, int src_y,
             int dst_x, int dst_y,
             int width, int height);
static unsigned int* blend_graymap_alloc
        (unsigned int fg, unsigned int bg,
         const unsigned char *src, int w, int h);
void gfx_text_size(int font, char *str, int *W, int *H);
void gfx_print(int flags, int font, int fg, int bg, int x, int y, 
               char *str,int *Rw, int *Rh);
int gfx_button(int flags,int x,int y, int w, int h, char *str, int action);
void gfx_button_enable(int btn,int a);
int gfx_label(int flags, int font, int fg, int bg,
              int x,int y, int w, int h, char *str);
void gfx_label_set_text(int flags,int labno, char *str);
void gfx_recsel_begin(int x,int y,int cox,int coy,int cow,int coh);
void gfx_recsel_end(int flags,int *Rx1, int *Ry1, int *Rx2, int *Ry2);
int gfx_text_entry(int flags, int x, int y, int w, int h, char *initstr);
char *gfx_text_entry_get(int no);
void gfx_text_entry_set(int flags, int no, char *str);
int gfx_scrollbar(int flags, int x,int y, int w,int h, 
                  double start, double end, double step,int action);
void gfx_scrollbar_set(int flags, int wno,
                       double start, double end, double step);
int gfx_dropdown(int x,int y,int w,int h,int action);
void gfx_dropdown_show(int wno,int show);
void gfx_dropdown_insert(int wno, int pos, const char *str, void *usr);
int gfx_dropdown_remove(int wno, int pos, void **Rusr);
void gfx_dropdown_select(int wno, int pos);
int gfx_dropdown_selected(int wno);
int gfx_dropdown_get(int wno, int pos, char **Rstr, void **Rusr);
int gfx_radio(int x,int y, int w,int h, const char *text,
              int action, int ident, int group);
void gfx_radio_show(int wno, int show);
void gfx_radio_set(int wno);
int gfx_radio_get(int wno);
int gfx_checkbox(int x,int y, int w,int h, const char *text, int action);
void gfx_checkbox_show(int wno, int show);
void gfx_checkbox_set(int wno,int value);
int gfx_checkbox_get(int wno);
int gfx_fwmenu(int font,int nentries, TUPiis_t *entries);
int gfx_push_screen();
void gfx_restore_screen(int sn);
int gfx_list(int flags, int x,int y,int w,int h, 
                int n_items, TUPscip_t *items, int click_action);
void gfx_list_redraw(int widget, int item);
void gfx_list_update_screen(int widget);
void gfx_list_delete(int flags, int widget, int pos, int N);
void gfx_list_insert(int flags,int widget, TUPscip_t *items, int pos, int N);
void gfx_list_replace(int flags, int widget, int pos, int N);

extern unsigned int gui_scrollbar_width, gui_scrollbar_height;


extern unsigned int  gfx_radio_width, gfx_radio_height;



void gfx_init_opengl(int x,int y,int w,int h, void (*func)(void *),void *arg);
void gfx_opengl_frame();


